package types

import "io"

// Status is the integer status code of a response.
type Status int

// Response contains the data in a response over the small web.
//
// Because protocols have so many differences, this type represents a
// greatest common denominator of request/response-oriented protocols.
type Response struct {
	// Status is the status code of the response.
	Status Status

	// Meta contains status-specific additional information.
	Meta any

	// Body is the response body, if any.
	Body io.Reader

	// Request is the request which generated this response.
	//
	// Available in clients.
	// Not set in servers, and there is no need to set it in server handlers.
	Request *Request
}

func (response *Response) Close() error {
	if cl, ok := response.Body.(io.Closer); ok {
		return cl.Close()
	}
	return nil
}

// ResponseReader is an object which can serialize a response to a protocol.
type ResponseReader interface {
	io.Reader
	io.WriterTo
	io.Closer
}
